--------------------------------------------------------------------------------
-- Title   : Detection block
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Signal detection
--------------------------------------------------------------------------------
-- File     : MIK_detection.vhd
-- Revision : x.x.x
-- Created  : April 28, 2015
-- Updated  : May 03, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2015 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_init.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity MIK_detection is
  port (
    p_reset      : in  std_logic;                      -- global reset
    p_clk60MHz   : in  std_logic;                      -- 60 MHz clock
    p_clk240MHz  : in  std_logic;                      -- 120 MHz clock
    p_pulseIn    : in  std_logic;                      -- input
    p_pulseWidth : in  std_logic_vector (31 downto 0); -- pulse width register
    p_csw        : in  std_logic;                      -- register chip select
    p_pulseOut   : out std_logic                       -- output
  );
end MIK_detection;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
-- TTL detection
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
architecture rtl_MIK_detection of MIK_detection is
  -----------------------------------------------------------------------------
  -- constant
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- signal
  -----------------------------------------------------------------------------
  signal s_resync : std_logic_vector (2 downto 0);
  -- s_risingEdge : std_logic;
  -- s_fallingEdge : std_logic;
--  signal s_cntPulseWidth    : unsigned (31 downto 0);
--  signal s_cntPulseWidthMax : unsigned (31 downto 0);
  signal s_cntPulseWidth    : unsigned (11 downto 0);
  signal s_cntPulseWidthMax : unsigned (11 downto 0);
  signal s_start   : std_logic;
  signal s_end     : std_logic;
  signal s_rstCNT  : std_logic;
  signal s_cntPrescaler  : unsigned (1 downto 0);
  signal s_prescaler_clk : std_logic;

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Resync input
  process (p_clk240MHz)
  begin
    if (rising_edge(p_clk240MHz)) then
      s_resync <= s_resync (1 downto 0) & p_pulseIn;
      -- s_risingEdge  <= s_resync(1) and not s_resync(2);
      -- s_fallingEdge <= s_resync(2) and not s_resync(1);
    end if;
  end process;

  -- Resync input
--  process (p_reset, p_clk240MHz)
--  begin
--    if (p_reset = '1') then
--      s_resync <= "000";
--    elsif (rising_edge(p_clk240MHz)) then
--      s_resync <= s_resync (1 downto 0) & p_pulseIn;
--      -- s_risingEdge  <= s_resync(1) and not s_resync(2);
--      -- s_fallingEdge <= s_resync(2) and not s_resync(1);
--    end if;
--  end process;

   -- Start
  process (p_reset, s_end, s_resync(2))
  begin
    if ((p_reset = '1') or (s_end ='1')) then
      s_start <= '0';
    elsif (rising_edge(s_resync(2))) then
      s_start <= '1';
    else
      s_start <= s_start;
    end if;
  end process;

  -- Clock pre-scaler
  process (s_start, p_clk240MHz)
  begin
    if (s_start <= '0') then
    s_cntPrescaler  <= "00";
    s_prescaler_clk <= '0';
    elsif (rising_edge(p_clk240MHz)) then
      if (s_cntPrescaler = "11") then
        s_cntPrescaler  <= "00";
        s_prescaler_clk <= '0';
      else
        s_cntPrescaler  <= s_cntPrescaler + 1;
        s_prescaler_clk <= not s_prescaler_clk;
      end if;
    end if;
  end process;

  -- Clock pre-scaler
--  process (p_reset, s_start, p_clk240MHz)
--  begin
--    if ((p_reset = '1') or (s_start <= '0')) then
--    s_cntPrescaler  <= "00";
--    s_prescaler_clk <= '0';
--    elsif (rising_edge(p_clk240MHz)) then
--      if (s_cntPrescaler = "11") then
--        s_cntPrescaler  <= "00";
--        s_prescaler_clk <= '0';
--      else
--        s_cntPrescaler  <= s_cntPrescaler + 1;
--        s_prescaler_clk <= not s_prescaler_clk;
--      end if;
--    end if;
--  end process;

  -- Clock pre-scaler
--  process (p_reset, p_clk240MHz)
--  begin
--    if (rising_edge(p_clk240MHz)) then
--      if ((p_reset = '1') or (s_start <= '0')) then
--      s_cntPrescaler  <= "00";
--      s_prescaler_clk <= '0';
--      elsif (s_cntPrescaler = "11") then
--        s_cntPrescaler  <= "00";
--        s_prescaler_clk <= '0';
--      else
--        s_cntPrescaler  <= s_cntPrescaler + 1;
--        s_prescaler_clk <= not s_prescaler_clk;
--      end if;
--    end if;
--  end process;

  -- Pulse generation
  process (p_reset, s_rstCNT, s_start, s_prescaler_clk)
  begin
    if ((p_reset = '1') or (s_rstCNT = '1') or (s_start = '0')) then
--      s_cntPulseWidth <= X"00000001";
      s_cntPulseWidth <= (OTHERS => '0');
      s_end <= '0';
      p_pulseOut <= '0';
    elsif (rising_edge(s_prescaler_clk)) then
      if (s_cntPulseWidth >= s_cntPulseWidthMax) then
        -- Start the gating windows
        s_cntPulseWidth <= s_cntPulseWidth;
        s_end <= '1';
        p_pulseOut <= '0';
      else
        s_cntPulseWidth <= s_cntPulseWidth + 1;
        s_end <= '0';
        p_pulseOut <= '1';
      end if;
    end if;
  end process;

  -- Latch from the register the pulse width
  process (p_reset, p_clk60MHz)
  begin
    if (p_reset = '1') then
      s_cntPulseWidthMax <= unsigned (c_wRegister_init(18) ((s_cntPulseWidthMax'LENGTH - 1) downto 0));
      s_rstCNT           <= '0';
    elsif (rising_edge(p_clk60MHz)) then
      if (p_csw = '1') then
        s_cntPulseWidthMax <= unsigned (p_pulseWidth  ((s_cntPulseWidthMax'LENGTH - 1) downto 0));
        s_rstCNT           <= '1';
      else
        s_cntPulseWidthMax <= s_cntPulseWidthMax;
        s_rstCNT           <= '0';
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end rtl_MIK_detection;
