--------------------------------------------------------------------------------
-- Title   : Hermes registers definitions
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : definition of the cPCI registers
--------------------------------------------------------------------------------
-- File     : cPCI_hermes_registers.vhdl
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : April 29, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--   Version 1.4.1 ; November 08 2013 ; Jean-Paul Ricaud
--     * Added a config read register (duplication, top-up, etc.)
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.type_lib.all;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package cPCI_hermes_registers is

  ------------------------------------------------------------------------------
  --Registers address
  ------------------------------------------------------------------------------
  constant c_add_boardid_reg  : std_logic_vector (9 downto 2) := "00000000"; -- board ID register
  constant c_add_firmware_reg : std_logic_vector (9 downto 2) := "00000001"; -- firmware's version register
  constant c_add_test_reg     : std_logic_vector (9 downto 2) := "00000010"; -- test register
  constant c_add_config_reg   : std_logic_vector (9 downto 2) := "00000011"; -- config register
  constant c_add_status_reg   : std_logic_vector (9 downto 2) := "00000100"; -- status register
  constant c_add_cmd_reg      : std_logic_vector (9 downto 2) := "00000101"; -- commands register
  constant c_add_foo_reg      : std_logic_vector (9 downto 2) := "00010100"; -- foo register, not used

  constant c_add_register : add_cpci_register_array := (
    "00000110",  -- register 0
    "00000111",  -- register 1
    "00001000",  -- register 2
    "00001001",  -- register 3
    "00001010",  -- register 4
    "00001011",  -- register 5
    "00001100",  -- register 6
    "00001101",  -- register 7
    "00001110",  -- register 8
    "00001111",  -- register 9
    "00010000",  -- register 10
    "00010001",  -- register 11
    "00010010",  -- register 12
    "00010011"); -- register 13

  ------------------------------------------------------------------------------
  -- Registers
  ------------------------------------------------------------------------------
  -- Write registers
  signal s_test_reg   : std_logic_vector (31 downto 0); -- test register
  signal s_status_reg : std_logic_vector (31 downto 0); -- status register
  signal s_cmd_reg    : std_logic_vector (31 downto 0); -- command register ; s_CSW = "000000000000001"

  -- wRegister 0  ; s_CSW = "000000000000010"
  -- wRegister 1  ; s_CSW = "000000000000100"
  -- wRegister 2  ; s_CSW = "000000000001000"
  -- wRegister 3  ; s_CSW = "000000000010000"
  -- wRegister 4  ; s_CSW = "000000000100000"
  -- wRegister 5  ; s_CSW = "000000001000000"
  -- wRegister 6  ; s_CSW = "000000010000000"
  -- wRegister 7  ; s_CSW = "000000100000000"
  -- wRegister 8  ; s_CSW = "000001000000000"
  -- wRegister 9  ; s_CSW = "000010000000000"
  -- wRegister 10 ; s_CSW = "000100000000000"
  -- wRegister 11 ; s_CSW = "001000000000000"
  -- wRegister 12 ; s_CSW = "010000000000000"
  -- wRegister 13 ; s_CSW = "100000000000000"
  signal s_wRegister : cpci_register_array;

  -- Read registers
  -- rRegister 0  ; CSR = "000000000000001"
  -- rRegister 1  ; CSR = "000000000000010"
  -- rRegister 2  ; CSR = "000000000000100"
  -- rRegister 3  ; CSR = "000000000001000"
  -- rRegister 4  ; CSR = "000000000010000"
  -- rRegister 5  ; CSR = "000000000100000"
  -- rRegister 6  ; CSR = "000000001000000"
  -- rRegister 7  ; CSR = "000000010000000"
  -- rRegister 8  ; CSR = "000000100000000"
  -- rRegister 9  ; CSR = "000001000000000"
  -- rRegister 10 ; CSR = "000010000000000"
  -- rRegister 11 ; CSR = "000100000000000"
  -- rRegister 12 ; CSR = "001000000000000"
  -- rRegister 13 ; CSR = "010000000000000"
  signal s_rRegister : cpci_register_array;

end package cPCI_hermes_registers;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
package body cPCI_hermes_registers is

end package body cPCI_hermes_registers;

