--------------------------------------------------------------------------------
-- Title   : Duplication block
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Signal duplication
--------------------------------------------------------------------------------
-- File     : dup_duplication.vhd
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : March 13, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity dup_duplication is
  port (
    p_inTTL    : in  std_logic;                     -- TTL input
    p_outTTL   : out std_logic_vector (4 downto 0); -- TTL outputs
    p_outPECL  : out std_logic_vector (4 downto 0)  -- LVPECL outputs
  );
end dup_duplication;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
-- TTL duplication
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
architecture rtl_dup_duplicationTTL of dup_duplication is
  -- constant

  -- signal
  signal s_delayed : std_logic_vector (4 downto 1);

  attribute keep : string;
  attribute keep of s_delayed : signal is "True";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  s_delayed(1) <= not p_inTTL;
  s_delayed(2) <= not p_inTTL;
  s_delayed(3) <= not p_inTTL;
  s_delayed(4) <= not p_inTTL;

  p_outTTL(1) <= not s_delayed(1);
  p_outTTL(2) <= not s_delayed(2);
  p_outTTL(3) <= not s_delayed(3);
  p_outTTL(4) <= not s_delayed(4);

  -- Unused signals
  p_outTTL(0) <= '0'; -- the IO is used as input
  p_outPECL   <= "00000";

end rtl_dup_duplicationTTL;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
-- LVPECL duplication
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--architecture rtl_dup_duplicationPECL of dup_duplication is
--
--  ------------------------------------------------------------------------------
--  --------------------------------- Main ---------------------------------------
--  ------------------------------------------------------------------------------
--  begin
--
--   p_outPECL(1) <= p_inTTL;
--   p_outPECL(2) <= p_inTTL;
--   p_outPECL(3) <= p_inTTL;
--   p_outPECL(4) <= p_inTTL;
--
--   -- Unused signals
--   p_outPECL(0) <= '0'; -- the IO is used as input
--   p_outTTL     <= "00000";
--
--end rtl_dup_duplicationPECL;
