--------------------------------------------------------------------------------
-- Title   : duplication testbench
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : duplication block test bench
--------------------------------------------------------------------------------
-- File     : dup_top_tb.vhd
-- Revision : x.x.x
-- Created  : November 05, 2012
-- Updated  : November 09, 2012
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity dup_top_tb is
end entity dup_top_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_dup_top_tb of dup_top_tb is

  -- constants
  constant Period_1kHz   : TIME := 100.000 ps;
  constant Period_500mHz : TIME := 500.000 ps;

  -- signals
  signal s_inTTL      : std_logic;                     -- input TTL
  signal s_outTTL     : std_logic_vector (4 downto 0); -- TTL outputs
  signal s_outPECL    : std_logic_vector (4 downto 0); -- LVPECL outputs
  signal s_reset      : std_logic;
  signal s_clk1kHz    : std_logic;
  signal s_clk500mHz  : std_logic;
  signal s_mon        : std_logic_vector (31 downto 0);
  signal s_led        : std_logic_vector (1 downto 0);

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  -- Reset
  DUT : entity work.dup_top (rtl_dup_top)
    port map (
      p_dup_inTTL      => s_inTTL,
      p_dup_outTTL     => s_outTTL,
      p_dup_outPECL    => s_outPECL,
      p_dup_clk1kHz    => s_clk1kHz,
      p_dup_clk500mHz  => s_clk500mHz,
      p_dup_RESET      => s_reset,
      p_dup_MON        => s_mon,
      p_dup_LED        => s_led
    );

  -- Clock generator
  process
  begin
    s_clk1kHz <= '0';
    wait for (Period_1kHz / 2);
    s_clk1kHz <= '1';
    wait for (Period_1kHz / 2);
  end process;

  process
  begin
    s_clk500mHz <= '0';
    wait for (Period_500mHz / 2);
    s_clk500mHz <= '1';
    wait for (Period_500mHz / 2);
  end process;

  process
  begin
    s_inTTL <= '0';
    s_reset <= '0';
    wait for 10 ns;
    s_reset <= '1';
    wait for 10 ns;
    s_reset <= '0';
    wait for 10 ns;

    wait for 57 ns;
    s_inTTL <= '1';
    wait for 55 ns;
    s_inTTL <= '0';
    wait for 50000 ns;
    s_inTTL <= '1';
    wait for 55 ns;
    s_inTTL <= '0';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_dup_top_tb;
