--------------------------------------------------------------------------------
-- Title   : reset testbench
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : reset block testbench
--------------------------------------------------------------------------------
-- File     : reset_tb.vhd
-- Revision : x.x.x
-- Created  : November 05, 2012
-- Updated  : November 05, 2012
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity reset_tb is
end entity reset_tb;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture archi_reset_tb of reset_tb is

  -- constants
  constant Period_60MHz  : TIME := 10000.000 ps;

  -- signals
  signal s_clk_60    : std_logic;
  signal s_rst_sr    : std_logic;
  signal s_csw       : std_logic; -- chip select of internal write transfert
  signal s_cmd_reg0  : std_logic; -- command register bit 0 ; s_CSW = "000000000000001"
  signal s_fpga_rst  : std_logic;
  signal s_board_rst : std_logic;

  component reset is
    port (
      -- Internal bus
      p_ext_RESETn : in std_logic;  -- external reset comming from the board reset chip
      p_clk60MHz   : in std_logic;  -- clock 60 MHz
      p_cmd_REG0   : in std_logic;  -- command register bit 0 ; CSW = "000000000000001"
      p_CSW        : in std_logic;  -- chip select of internal write transfert
      -- Other signals
      p_fpga_RST   : out std_logic; -- internal reset of the FPGA (process)
      p_board_RST  : out std_logic  -- mother board global reset
    );
  end component reset;

  ----------------------------------------------------------------------------
  ----------------------------------------------------------------------------
  begin

  -- Reset
  DUT : reset
  port map (
    EXT_RESETn => s_rst_sr,
    CLK60MHz   => s_clk_60,
    CMD_REG0   => s_cmd_reg0,
    CSW        => s_csw,
    FPGA_RST   => s_fpga_rst,
    BOARD_RST  => s_board_rst
  );

  -- Clock generator
  process
  begin
    s_clk_60 <= '0';
    wait for (Period_60MHz / 2);
    s_clk_60 <= '1';
    wait for (Period_60MHz / 2);
  end process;

  process
  begin
    s_cmd_reg0 <= '0';
    s_csw <= '0';

    -- Init states
    s_rst_sr <= '1';
    wait for 117 ns;
    s_rst_sr <= '0';
    wait for 125 ns;
    s_rst_sr <= '1';

    -- Software reset
    wait for 50 ns;
    s_cmd_reg0 <= '1';
    s_csw <= '1';
    wait for 50 ns;
    s_csw <= '0';

    -- *** END ***
    wait;
  end process;

  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------

end architecture archi_reset_tb;

